@@ -1,3 +1,5 @@
+import os
+
 FOCAL_LOSS_GAMMA = 0.0
 FOCAL_LOSS_GAMMA_NEG_POS = 0.25
 FOCAL_LOSS_BETA_NEG_POS = 1.0
@@ -17,3 +19,6 @@ DATA_ACTION_NO_NEED_LOAD_EMB_M = "DATA_ACTION_NO_NEED_LOAD_EMB_M"
 
 NEG_RATIO = 1 - 0.05897253769515213
 Data_Folder = '/home/'
+
+RIPDB = os.environ.get("RIPDB") == 'true'
+DEBUG = os.environ.get("DEBUG") == 'true'
