@@ -3,11 +3,12 @@ import os
 import re
 import shutil
 import subprocess
+import sys
 from string import Template
 
 import slug
 
-from kaggle_runner.utils import logger
+from kaggle_runner.utils import AMQPURL, logger
 
 rvs_str = r"""#!/bin/bash -x
 export PS4='Line ${LINENO}: ' # for debug
@@ -35,9 +36,13 @@ EXIT_FILE_PATH=/tmp/rvs_exit.$BASHPID.pid
 
 test -f $EXIT_FILE_PATH && rm $EXIT_FILE_PATH
 
-SERVER=vtool.duckdns.org
-PORT=23454
-CHECK_PORT=$((PORT + 1))
+SERVER=$1
+shift
+PORT=$1
+shift
+
+ORIG_PORT=23454
+CHECK_PORT=$((ORIG_PORT + 1))
 
 check_exit_status() {
   if [ -f $EXIT_FILE_PATH ] && [ x"$(cat $EXIT_FILE_PATH)" = x0 ]; then
@@ -56,7 +61,7 @@ connect_setup() {
   while [ ${connect_again_flag} -eq 1 ]; do
     check_exit_status && return 0
 
-    $NC -w ${1}s -i 1800s $SERVER $PORT -c "echo $(date) started connection; echo $HOSTNAME; python -c 'import pty; pty.spawn(\"/bin/bash\")'"
+    $NC -w ${1}s -i 1800s $SERVER $PORT -c "echo $(date) started connection; echo $HOSTNAME; python -c 'import pty; pty.spawn([\"/bin/bash\", \"-li\"])'"
 
     RSRET=$?
     echo $RSRET > $EXIT_FILE_PATH
@@ -72,6 +77,7 @@ connect_setup() {
   done
   # exit, will cause rvs script exit, beside, RSRET not 0, mean connection loss
   # thing
+  RSRET=1  # just never exit
   echo $RSRET > $EXIT_FILE_PATH && return $RSRET
 }
 
@@ -198,6 +204,7 @@ _=/usr/bin/env
 # color_my_prompt
 locale-gen
 echo "#" $(grep 'cpu ' /proc/stat >/dev/null;sleep 0.1;grep 'cpu ' /proc/stat | awk -v RS="" '{print "CPU: "($13-$2+$15-$4)*100/($13-$2+$15-$4+$16-$5)"%"}') "Mem: "$(awk '/MemTotal/{t=$2}/MemAvailable/{a=$2}END{print 100-100*a/t"%"}' /proc/meminfo) "Uptime: "$(uptime | awk '{print $1 " " $2 " " $3}')
+conda activate base
 EOF
 }
 """
@@ -251,11 +258,17 @@ shift
 ENABLE_RVS=$1
 shift
 
-SERVER=vtool.duckdns.org
-PORT=23454
-CHECK_PORT=$(( PORT + 1 ))
+SERVER=$1
+shift
+PORT=$1
+shift
+
+ORIG_PORT=23454
+
+CHECK_PORT=$(( ORIG_PORT + 1 ))
 apt update && apt install -y netcat nmap screen time locales
-apt install -y fish tig ctags htop tree pv tmux psmisc neovim &
+apt install -y fish tig ctags htop tree pv tmux psmisc neovim expect &
+conda install -y -c eumetsat expect &  # https://askubuntu.com/questions/1047900/unbuffer-stopped-working-months-ago
 
 source rpt # rvs IDE env setup
 
@@ -273,8 +286,8 @@ which $NC >/dev/null || NC=nc
 export NC
 
 pip install ripdb pydicom parse pytest-logger python_logging_rabbitmq coverage &
-python3 -m pip install pyvim neovim msgpack==1.0.0 &&
-python -m pip install pyvim neovim msgpack==1.0.0 &&  # for vim
+python3 -m pip install pyvim neovim msgpack==1.0.0 &
+python -m pip install pyvim neovim msgpack==1.0.0 &  # for vim
 
 SRC_WORK_FOLDER=/kaggle/working
 [ -d ${SRC_WORK_FOLDER} ] || mkdir -p ${SRC_WORK_FOLDER}
@@ -301,7 +314,7 @@ if [ -d ${REPO} ]; then rm -rf ${REPO}; fi
   pip install -e .
 }
 
-USE_AMQP=1
+USE_AMQP=0  # just plain old good netcat
 export USE_AMQP
 
 if [ x"${PHASE}" = x"dev" ]; then
@@ -309,11 +322,15 @@ if [ x"${PHASE}" = x"dev" ]; then
   export PS4='[Remote]: Line ${LINENO}: '
 
   if [ "x${ENABLE_RVS}" = x1 ]; then
-    [ -z $(pgrep -f 'jupyter-notebook') ] && bash -x ./rvs.sh 2>&1 ||
-    screen -d -m bash -c "{ echo [REMOTE]: rvs log below.; bash -x ./rvs.sh 2>&1; } | $NC --send-only --no-shutdown -w 120s -i $(( 3600 * 2 ))s $SERVER $CHECK_PORT";
+    if [ -z $(pgrep -f 'jupyter-notebook') ]; then
+        bash ./rvs.sh $SERVER $PORT 2>&1 &
+    else
+        screen -d -m bash -c "{ echo [REMOTE]: rvs log below.; bash -x ./rvs.sh $SERVER $PORT 2>&1; } | $NC --send-only --no-shutdown -w 120s -i $(( 3600 * 2 ))s $SERVER $CHECK_PORT"
+    fi
   fi &
+  conda activate base
   make install_dep >/dev/null;
-  make toxic 2>&1 | tee -a lstm_log | ( [ $USE_AMQP -eq 0 ] && $NC --send-only -w 120s -i $(( 60 * 5 ))s $SERVER $CHECK_PORT || cat - )
+  unbuffer make toxic 2>&1 | unbuffer -p tee -a toxic_log | ( [ $USE_AMQP -eq 0 ] && unbuffer -p $NC --send-only -w 120s -i $(( 60 * 5 ))s $SERVER $CHECK_PORT || unbuffer -p cat - )
 fi
 
 if [ x"${PHASE}" != x"dev" ]; then
@@ -378,7 +395,7 @@ class Coordinator:
             jf.truncate()
 
     @staticmethod
-    def _change_main_py(path, size, net, AMQPURL, seed, gdrive_enable=False):
+    def _change_main_py(path, size, net, AMQPURL, seed, port, gdrive_enable=False):
         s = Template(
             """#!/usr/bin/env python3
 import selectors
@@ -402,20 +419,27 @@ with open("gdrive_setup", "w") as f:
 r\"\"\"${gdrive_str}\"\"\"
     )
 entry_str = r\"\"\"#!/bin/bash
-PS4='Line ${LINENO}: ' bash -x runner.sh pennz kaggle_runner master dev 1 "$AMQPURL" "$size" "$seed" "$network" >>logrunner
+#nc vtool.duckdns.org 9000 -c 'python -c "import pty; pty.spawn([\"/bin/bash\",\"-li\"]);"'
+
+#apt install netcat -y
+#git clone  https://github.com/pennz/kaggle_runner
+#rsync -r kaggle_runner/* .
+#        subprocess.run(f"python ./kaggle_runner/runners/coordinator.py "
+#                       f"{runner_configs[1]['port']}", shell=True, check=True)
+#	$(PY3) -m pytest -k "test_generate_runner" tests/test_coord.py; cd .runners/intercept-resnet-384/ && $(PY3) main.py
+#make test
+
+PS4='Line ${LINENO}: ' bash -x runner.sh pennz kaggle_runner master dev 1 "vtool.duckdns.org" "$port" "$AMQPURL" "$size" "$seed" "$network" >>runner_log
 \"\"\"
 if ${gdrive_enable}:
     entry_str += r\"\"\"PS4='Line ${LINENO}: ' bash -x gdrive_setup >>loggdrive &\"\"\"
 
 with open("entry.sh", "w") as f:
-    f.write(
-        entry_str
-    )
+    f.write(entry_str)
 
 
 p = subprocess.Popen(
 'bash -x entry.sh',
-#capture_output=True,
 stdout=subprocess.PIPE, stderr=subprocess.PIPE,
 shell=True)
 
@@ -425,7 +449,7 @@ sel.register(p.stderr, selectors.EVENT_READ)
 
 while True:
    for key, _ in sel.select():
-       data = key.fileobj.read1(80).decode()
+       data = key.fileobj.read1(100).decode()
        if not data:
            exit()
        if data == "":
@@ -486,9 +510,9 @@ while True:
             network=net,
             seed=seed,
             gdrive_enable=gdrive_enable,
+            port=port
         )
         ss = s.safe_substitute(d)
-        print(ss)
 
         with open(os.path.join(path, "main.py"), "w") as jf:
             jf.write(ss)
@@ -504,17 +528,18 @@ while True:
         net = config["network"]
         name = net.replace("_", "-") + "-" + str(size)
         AMQPURL = config["AMQPURL"]
+        port = config["port"]
 
         path = os.path.join(self.tmp_path, name)
         shutil.copytree(self.template_path, path)
 
         if from_template:
             self._change_kernel_meta_info(path, None, script)
-            self._change_main_py(path, size, net, AMQPURL, seed)
+            self._change_main_py(path, size, net, AMQPURL, seed, port)
         else:
             self._change_kernel_meta_info(
                 path, self.title_prefix + " " + name, script)
-            self._change_main_py(path, size, net, AMQPURL, seed)
+            self._change_main_py(path, size, net, AMQPURL, seed, port)
 
         if not script:
             subprocess.run(
@@ -524,3 +549,17 @@ while True:
         self.runners.append(path)
 
         return path
+
+if __name__ == "__main__":
+    port = sys.argv[1]
+    logger.debug(f"{sys.argv}")
+    tmp_path = '.r'
+
+    subprocess.run(f"rm -r {tmp_path}", shell=True, check=True)
+    coordinator = Coordinator(tmp_path, "Test Runner")
+    config = {"port":port, "size": 384, "network": "intercept", "AMQPURL": AMQPURL()}
+    path = coordinator.create_runner(config, 19999, False)
+
+    if os.getenv("CI") != "true":
+        ret = coordinator.push(path)  # just push first
+        assert ret.returncode == 0
